#! /usr/bin/perl

# $Id: main.pl,v 2.11 2003/05/12 12:52:54 blusseau Exp $

use strict;
use Locale::gettext;
use POSIX;     # Needed for setlocale()

use vars qw(%GLOBAL_VARS);

require "functions.pl"; # Some usefull functions

my $script_dir="";
my $script_conf_file;

my %handlers;

$|=1; # Autoflush

# Add the path of main.pl script to the Perl libraries path
BEGIN {
	my ($libdir) = $0 =~m|(.*)/|; # extract path from $0
	push (@INC,$libdir) unless grep {/$libdir/} @INC;
};

#*****************************************
# load and interpret the scripts.conf file
#*****************************************
sub load_config {
	%GLOBAL_VARS={};
	my @modules;
	if (open(CONF,$script_conf_file)) {
	 	while (<CONF>) {
			chomp;
			s/^\s+//g;
			next if ((/^\#/) or (not $_));
			# print "$_\n";
		  SWITCH: {
				s/^set\s+// && do {
					my ($var,$value)=split(/\s+/);
					$value=unquote($value);
					my $str=qq(\$GLOBAL_VARS{$var}="$value";);
					eval $str;
					last SWITCH;
				};
				s/^source\s+// && do {
					my $file=$_;
					$file="$script_dir/$file" unless ($file=~/^\//);
					unless (grep {/$file/} @modules) {
						delete $INC{$file} if $INC{$file}; # so we can reload the file with require
						if (! -f $file) {
							print STDERR sprintf("*** "._("Perl Init: Can't source %s").":"._("No such file").".\n",$file);
						} else {
							eval {
								require $file; # try to source the script
							};
							if ($@) {
								print STDERR sprintf("*** "._("Perl Init: Can't source %s").":\n$@",$file);
							}
							push @modules,$file;
						}
					} else {
						print STDERR sprintf("*** "._("Perl Init: Script %s already loaded").".\n",$file);
											}
					last SWITCH;
				};
			}
		}
		close(CONF);
	}
}

#***********************************************************************
# initialize some variables just after the interpreter has been loaded
#***********************************************************************
# parameter: 1 string: the script dir
sub dchub_perl_init {
	$script_dir=$_[0];
	
	# For localisation
	bindtextdomain("dchub_handlers","$script_dir/i18n");
	textdomain("dchub_handlers");

	my $locale=dchub::db_get("LANGUAGE");
	if (defined $locale) {
		setlocale(LC_ALL, "$locale");
		print "Perl: ".sprintf(_("Using %s as locale for perl handlers"),$locale).".\n";
	}

	$script_conf_file="$script_dir/dchub_scripts.conf";
	load_config;
}

#***************************
# add a hook to an handler *
#***************************
# parameter: 1 string: name of the event (ex: myinfo)
#					   (see the Documentation/script for the list of events)
# 			 2 reference to a sub containing the code to be execute when the event is fire
sub add_hook ($&) {
	my $event=shift;
	my $function=shift;
	# print "PUSH:[$event -> $function\n";
	push @{$handlers{$event}},$function;
}

#***********************************************************************
# Catch all invalid method calls
# the function tries to define an handler to call all the hooks
# that are associated with it.
#***********************************************************************
sub AUTOLOAD {
	local *FH;
	our ($AUTOLOAD);

	# print "AUTOLOAD $AUTOLOAD\n";
	
	my $event=$AUTOLOAD;
	if ($event=~s/^.*::dchub_(.*)_handler/$1/) {
		my $funcname="dchub_${event}_handler";
		if (not defined $handlers{$funcname}) {
			my $eval=qq(sub $funcname(\\%) { my \$array=\$handlers{$event};).
						q(
						  foreach my $refsub (@$array) {
							  &$refsub(@_);
						  }
					  });
			eval $eval;
			no strict 'refs';
			&$funcname(@_);
		}
	} else {
		$event=~s/^main:://;
		print STDERR "*** ".sprintf(_("Unknown Perl function:%s"),$event).".\n";
	}
}

#***********************************************************************
# undefined all autoloaded perl handlers
#***********************************************************************
sub dchub_perl_clear_autoloaded_handlers {
	no strict 'refs';
	foreach my $function (keys %handlers) {
		# print "Clear:$function\n";
		delete $handlers{$function};
	}
	load_config;
	my $locale=dchub::db_get("LANGUAGE");
	if (defined $locale) {
		setlocale(LC_ALL, "$locale");
		print "Perl: ".sprintf(_("Using %s as locale for perl handlers"),$locale).".\n";
	}
}

#******************************************************************************
#******************************************************************************
#  Event handler entry point
#******************************************************************************
#******************************************************************************
# parameter: 1 hash table (evt_array)
sub dchub_perl_handler {
  my $evt_array = $_[0];	# evt_array is a reference to a hash table (%) containing all the parameters
							# the event array contains at least the following entries
							# "event": the name of the event
							# "nickname": the nickname of the user generating this event
							# "argc": the number of optionnal arguments
							# if optionnal arguments exist, they are named "0", "1", "2", "3, ...
	
  my $hdl_name="dchub_".$$evt_array{"event"}."_handler";

  # call the handle which must deal with this event
  no strict 'refs';
  eval {
      &$hdl_name($evt_array); # Call the sub (AUTOLOAD if undef)
  };
  if ($@) {
  	  print _("*** Perl Script ERROR executing the handler")." $hdl_name:\n$@";
  }
}

#***********************************************************************
# Extended globalchat handler that extract nickname and message
# from global chat. This function also translate some forbidden protocol
# char like | and $
#***********************************************************************
sub dchub_globalchat_handler(\%) {
	my $evt_array = $_[0];
	my $nickname=$evt_array->{"nickname"};
	my $msg=$evt_array->{0};
	$msg=~s/<\Q$nickname\E>\s*//;
	$msg=~s/\|+$//;
	my $array=$handlers{globalchat};
	foreach my $refsub (@$array) {
		&$refsub($evt_array,$nickname,unescape($msg));
	}
}

#***********************************************************************
# Extended privchat handler that extract sender, receiver and message
# from private chat. This function also translate some forbidden protocol
# char like | and $
#***********************************************************************
sub dchub_privchat_handler(\%) {
	my $evt_array = $_[0];
	my $from=$evt_array->{"nickname"};
	my $to=$evt_array->{0};
	my $msg=$evt_array->{1};
	$msg=~s/<\Q$from\E>\s*//;
	$msg=~s/\|+$//;
	my $array=$handlers{privchat};
	foreach my $refsub (@$array) {
		&$refsub($evt_array,$from,$to,unescape($msg));
	}
}

#******************************************************************************
# sub to avoid writing gettext
sub _(@) { gettext(@_); }
